/*
 * CDTOOL is a program to play and catalog audio compact discs on
 * a cdrom drive
 *
 * Copyright 1994 Thomas Insel.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 *
 * For information on CDTOOL, alternate licensing, or whatever, 
 * contact Thomas Insel via email (tinsel@uiuc.edu) or write to
 * Thomas Insel, 210 Parkview Drive, Bloomington, IL 61701-2038.
 * 
 * Multiple cdrom drive support
 * cdinfo command
 * skip track option
 * some minor enhanchments
 * and some bugs fixed
 * by
 * Sven Oliver Moll
 * 
 * Copyright 1995,96 Sven Oliver Moll
 */

#include "config.h"
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <strings.h>
#include <sys/stat.h>

#include "util.h"
#include "commands.h"
#include "shuffle.h"
#include "hardware.h"
#include "info.h"
#include "cdtool.h"

static char *svnid = "$Id: cdtool.c 797 2004-09-12 15:38:04Z max $";
void do_usage(char *progname);

/************************************************************************/
/* Procedure:  main
 * Purpose:    C code main function
 * 
 * Inputs:     command line args
 * Outputs:    to stdout
 * Returns:    error code (0 if exit is OK)
 * Notes:  
 *   1.
 */
/************************************************************************/
int main(int argc, char *argv[]) 
{
  cdtool_t      cdtool;
  int		info=I_AUDIOSTATUS, error=0, opt, cdfile, trk0=0, trk1=0;
  extern int	optind, getopt();
  extern char	*optarg;

  int  usedb = 1;
  char p_format = P_LONG;
  int  iRepeat = 1;
  int  iTracks = 100;
  int  iCDNum = 0;

  cdtool.progname = "unknown";
  cdtool.cd_device = NULL;
  cdtool.cd_device_hint = NULL;
  
  cdtool_show_debug = FALSE;
  cdtool_show_verbose = FALSE;
  cdtool_show_crlf = DOLF;
  
  if ((cdtool.progname = rindex(argv[0],'/')) != 0)
    cdtool.progname++;
  else
    cdtool.progname = argv[0];

  if ((cdtool.cd_device_hint = getenv("CDTOOLDEV")) == NULL) 
    cdtool.cd_device_hint = strdup(CD_DEVICE);

  while((opt = getopt(argc,argv,"0123456789?ad:Dhi:nrstvVp:R:T:")) != EOF)
    {
      switch(opt)
	{    
	 case '0':
	 case '1':
	 case '2':
	 case '3':
	 case '4':
	 case '5':
	 case '6':
	 case '7':
	 case '8':
	 case '9':
	  cdtool.cd_device_hint = strdup(CD_DEVICE "?");
	  *(cdtool.cd_device_hint + sizeof(CD_DEVICE)-1) = opt;
	  iCDNum = opt - '0';
	  break;
	 case 'a':
	  error = 1;
	  if ( strcmp(cdtool.progname, "cdinfo") == 0 )
	    {
	      error = 0;
	      info = I_DISP_ABS;
	    }
	  break;
	 case 'd':
	  cdtool.cd_device_hint=optarg;
	  break;
	 case 'n':
	  error = 1;
	  if ( strcmp(cdtool.progname, "cdir") == 0 )
	    {
	      error = 0;
	      usedb = 0;
	    }
	  break;
	 case 'r':
	  error = 1;
	  if ( strcmp(cdtool.progname, "cdir") == 0 )
	    {
	      error = 0;
	      p_format = P_RAW; 
	    }
	  if ( strcmp(cdtool.progname, "cdinfo") == 0 )
	    {
	      error = 0;
	      info = I_DISP_REL;
	    }
	  break;
	 case 's':
	  error = 1;
	  if ( strcmp(cdtool.progname, "cdir") == 0 )
	    {
	      error = 0;
	      p_format = P_QUICK;
	    }
	  if ( strcmp(cdtool.progname, "cdinfo") == 0 )
	    {
	      error = 0;
	      info = I_AUDIOSTATUS;
	    }
	  break;
	 case 't':
	  error = 1;
	  if ( strcmp(cdtool.progname, "cdir") == 0 )
	    {
	      error = 0;
	      p_format = P_TEMPL;
	    }
	  if ( strcmp(cdtool.progname, "cdinfo") == 0 )
	    {
	      error = 0;
	      info = I_TRACK;
	    }
	  break;

	  /* verbose info printout */
	 case 'v':
	  cdtool_show_verbose = TRUE;
	  error = 1;
	  if (( strcmp(cdtool.progname, "cdshuffle") == 0 ) ||
	      ( strcmp(cdtool.progname, "cdvolume") == 0 ) ||
	      ( strcmp(cdtool.progname, "cdplay") == 0 ) ||
	      ( strcmp(cdtool.progname, "cdstart") == 0 )) {
	      error = 0;
	  }
	  else if ( strcmp(cdtool.progname, "cdir") == 0 ) {
	      error = 0;
	      usedb = 1;
	  }
	  else if ( strcmp(cdtool.progname, "cdinfo") == 0 ) {
	      error = 0;
	      info = I_DISP_ALL;
	  }
	  break;

	  /* for debug, overload the program name, e.g., */
	  /* cdtool -p info is the same as cdinfo */
	case 'p':
	  cdtool.progname = optarg;
	  error = 0;
	  break;

	case 'R':
	  if (( strcmp(cdtool.progname, "cdshuffle") == 0 ) ||
	      ( strcmp(cdtool.progname, "cdstart") == 0 ) ||
	      ( strcmp(cdtool.progname, "cdplay") == 0 )) {
	      error = 0;
	      iRepeat = atoi(optarg);
	  }
	  break;

	case 'T':
	  if (( strcmp(cdtool.progname, "cdshuffle") == 0 ) ||
	      ( strcmp(cdtool.progname, "cdstart") == 0 ) ||
	      ( strcmp(cdtool.progname, "cdplay") == 0 )) {
	      error = 0;
	      iTracks = atoi(optarg);
	  }
	  break;
	  
         case 'D':
             /* enable debugging */
             cdtool_show_debug = TRUE;
             break;

	 /* print version number and then standard HELP */
	 case 'V':
	     printf ("%s %s\n", cdtool.progname, VERSION_STRING);
	     printf ("Build: %s %s [%s]\n",__DATE__,__TIME__, svnid);
             exit(0);

	  /* general options */
	 case '?':
	 case 'h':
	 default:
	  error = 1;
	  break;
	}
    }

  if (error)
    {
      do_usage(cdtool.progname);
      exit(1);
    }

  if ((cdtool.cd_device = device_find(cdtool.cd_device_hint)) == NULL) 
    {
      ERRORMSG("%s: no usable cdrom found. Try passing -d", cdtool.progname);
      exit(1);
    }
  
  if ( (cdfile=open(cdtool.cd_device, O_RDONLY | O_NONBLOCK)) == -1 ) 
    {
      cdhw_t hw;
      int    iResult;
      ERRORMSG("%s: can't open cdrom (%s)",cdtool.progname,cdtool.cd_device);
      ERRORMSG("%s: error: %s", cdtool.progname, strerror(errno));

      iResult = cd_status(&cdtool, &hw, -1);
      ERRORMSG("%s:  CD status is %s", cdtool.progname,
        cd_status_text(iResult));
      exit(1);
    }

  if (( strcmp(cdtool.progname, "cdplay") == 0 ) ||
      ( strcmp(cdtool.progname, "cdstart") == 0 ))
    {
	if ( optind < argc ) {
	    trk0=atoi(argv[optind++]);
            DEBUGMSG("main: trk0=%d", trk0);
	}
	if ( optind < argc ) {
	    trk1=atoi(argv[optind]);
            DEBUGMSG("main: trk1=%d", trk0);
	}
	optind--;
	if (strcmp(argv[optind],"-") == 0 ) {
	    trk0=SKIPBACK;
            DEBUGMSG("main: trk0=SKIPBACK(%d)", trk0);
	}
	if (strcmp(argv[optind],"+") == 0 ) {
	    trk0=SKIPFORWARD;
            DEBUGMSG("main: trk0=SKIPFORWARD(%d)", trk0);
	}
	if (strcmp(argv[optind],"?") == 0 ) {
	    do_usage(cdtool.progname);
	    exit(0);
	}
	if ((strcmp(argv[optind],"start") == 0) ||
	     (strcmp(argv[optind],"x") == 0 ) ||
	     (strcmp(argv[optind],"0") == 0 )) {
	    trk0=SKIPSTART;
            DEBUGMSG("main: trk0=SKIPSTART(%d)", trk0);
        }

	if (strcmp(argv[optind],"stop") == 0) {
            DEBUGMSG("main: stop");
            do_stop(&cdtool, cdfile, iCDNum);
	    close(cdfile);
	    exit(0);
        }
	if (strcmp(argv[optind],"pause") == 0) {
            DEBUGMSG("main: pause");
            do_pause(&cdtool, cdfile);
	    close(cdfile);
	    exit(0);
        }
	if (strcmp(argv[optind],"eject") == 0) {
            DEBUGMSG("main: eject");
            do_eject(&cdtool, cdfile);
	    close(cdfile);
	    exit(0);
        }
	if (strcmp(argv[optind],"close") == 0) {
            DEBUGMSG("main: close");
            do_close(&cdtool, cdfile);
	    close(cdfile);
	    exit(0);
        }
	if (strcmp(argv[optind],"info") == 0) {
            DEBUGMSG("main: info");
            do_info(&cdtool, cdfile, I_DISP_ALL);
	    do_crlf(stdout);
	    close(cdfile);
	    exit(0);
        }

	if (strcmp(argv[optind],"dir") == 0) {
            DEBUGMSG("main: dir");
            do_dir(&cdtool, cdfile, P_RAW, 0);
	    close(cdfile);
	    exit(0);
        }
	if (strcmp(argv[optind],"resume") == 0) {
            DEBUGMSG("main: resume");
	    do_play(&cdtool, cdfile, 0, 0, NULL);
        }

	if (strcmp(argv[optind],"shuffle") == 0) {
            DEBUGMSG("main: shuffle");
            do_shuffle(&cdtool, cdfile, iCDNum, iTracks, iRepeat);
        }
	if (strcmp(argv[optind],"volume") == 0) {
	     if(++optind < argc) {
		  int vol=atoi(argv[optind]);
                  DEBUGMSG("main: volume");
		  exit( do_volume(&cdtool, cdfile, vol) );
	     }
	}

	/* block to play a CD if it is an audio cd */
	{
	    cdhw_t *hw;
	    int    iTemp;
	    hw = check_disc (&cdtool, cdfile, &iTemp);
	    if (iTemp == TRUE)
		do_play(&cdtool, cdfile, trk0, trk1, hw);
	    free_hw_buf(hw);
	}
    }

  else if ( strcmp(cdtool.progname, "cdstop") == 0 )
    do_stop(&cdtool, cdfile, iCDNum);
  
  else if ( strcmp(cdtool.progname, "cdeject") == 0 )
    do_eject(&cdtool, cdfile);

  else if ( strcmp(cdtool.progname, "cdclose") == 0 )
    do_close(&cdtool, cdfile);

  else if ( strcmp(cdtool.progname, "cdreset") == 0 )
    do_reset(&cdtool, cdfile);

  else if ( strcmp(cdtool.progname, "cdpause") == 0 )
    do_pause(&cdtool, cdfile);

  else if ( strcmp(cdtool.progname, "cdshuffle") == 0 )
    do_shuffle(&cdtool, cdfile, iCDNum, iTracks, iRepeat);

  else if ( strcmp(cdtool.progname, "cdinfo") == 0 ) {
    do_info(&cdtool, cdfile, info);
    do_crlf(stdout);
  }

  else if ( strcmp(cdtool.progname, "cdir") == 0 )
    do_dir(&cdtool, cdfile, p_format, usedb);

  else if ( strcmp(cdtool.progname, "cdvolume") == 0 ) {
    if ( optind < argc ) {
      int vol=atoi(argv[optind]);
      exit( do_volume(&cdtool, cdfile, vol) );
    }
    else
      do_usage(cdtool.progname);
  }
  else do_usage(cdtool.progname);

  close(cdfile);

  exit(0);
}

/************************************************************************/
/* Procedure:  do_usage
 * Purpose:    to print usage info (help)
 * 
 * Inputs:     program name
 * Outputs:    to stderr
 * Returns:    n/a
 * Notes:  
 *   1.
 */
/************************************************************************/
void do_usage(char *progname)
{
  fprintf(stderr, "%s", VERSION_STRING); fprintf(stderr,"\n");
  
  if (( strcmp(progname, "cdplay") == 0 ) ||
      ( strcmp(progname, "cdstart") == 0 )) {

    if ( strcmp(progname, "cdplay") == 0 )
      fprintf(stderr, "usage: cdplay [start [stop]]\n");
    else
      fprintf(stderr, "usage: cdstart [start [stop]]\n");

    fprintf(stderr, " +             next track\n");
    fprintf(stderr, " -             previous track\n");
    fprintf(stderr, " x             restart at track 1\n");
    fprintf(stderr, " S             play track S\n");
    fprintf(stderr, " S E           play track S..E\n");
    fprintf(stderr, " start         play from track 1\n");
    fprintf(stderr, " stop          stop playing\n");
    fprintf(stderr, " resume        resume playing\n");
    fprintf(stderr, " eject         eject CD-ROM\n");
    fprintf(stderr, " close         close CD-ROM tray\n");
    fprintf(stderr, " info          display info\n");
    fprintf(stderr, " dir           display directory\n");
    fprintf(stderr, " shuffle       shuffle play\n");
    fprintf(stderr, " -v            verbose output\n");
    fprintf(stderr, " -R rr         times to repeat the CDROM (shuffle only)\n");
    fprintf(stderr, " -T tt         max tracks to play (shuffle only)\n");
  }
  
  else if ( strcmp(progname, "cdshuffle") == 0 ) {
    fprintf(stderr, "usage: cdshuffle [options]\n");
    fprintf(stderr, " -R rr         times to repeat the CDROM\n");
    fprintf(stderr, " -T tt         max tracks to play\n");
    fprintf(stderr, " -v            verbose output\n");
  }
  else if ( strcmp(progname, "cdstop") == 0 )
    fprintf(stderr, "usage: cdstop  [options]\n");
  else if ( strcmp(progname, "cdeject") == 0 )
    fprintf(stderr, "usage: cdeject  [options]\n");
  else if ( strcmp(progname, "cdclose") == 0 )
    fprintf(stderr, "usage: cdclose  [options]\n");
  else if ( strcmp(progname, "cdvolume") == 0 ) 
      fprintf(stderr, "usage: cdvolume [0-255]\n");
  else if ( strcmp(progname, "cdpause") == 0 )
    {
      fprintf(stderr, "usage: cdpause  [options]\n");
      fprintf(stderr, "  resume play with cdplay/cdstart\n");
    } 
  else if ( strcmp(progname, "cdir") == 0 ) 
    {
      fprintf(stderr, "usage: cdir [ [-n] {-s|-t} ]\n");
      fprintf(stderr, " -n            listing without titles\n");
      fprintf(stderr, " -r            raw read of toc\n");
      fprintf(stderr, " -s            short listing (title/artist/track)\n");
      fprintf(stderr, " -t            print template for database entry\n");
    } 
  else if ( strcmp(progname, "cdinfo") == 0 ) 
    {
      fprintf(stderr, "usage: cdinfo [ {-a|-r|-s|-t} ]\n");
      fprintf(stderr, " -a            print absolute disc time\n");
      fprintf(stderr, " -r            print relative track time\n");
      fprintf(stderr, " -s            print audiostatus\n");
      fprintf(stderr, " -t            print current track\n");
      fprintf(stderr, " -v            print status, track, absolute, relative\n");
    }
  else 
    fprintf(stderr, "Please call as cdplay/cdstart, cdstop, cdeject, cdclose,\n\
cdvolume, cdpause, cdir, cdinfo, or cdshuffle\n");

  /* standard options below here: */
  fprintf(stderr, " -V            build, version, and this help\n");
  fprintf(stderr, " -? -h         print this message\n");
  fprintf(stderr, " -[0-9]        use device "CD_DEVICE"[0-9]\n");
  fprintf(stderr, " -d dev        use device dev\n");
  fprintf(stderr, " -D            enable debugging information\n");
}
